@@ -178,8 +178,6 @@ static int display_indirect(struct iinfo *ind, int indblocks, int level,
 				print_gfs2("   ");
 		}
 		print_gfs2("%d: 0x%"PRIx64" => ", pndx, ind->ii[pndx].ptroff);
-		if (termlines)
-			move(line,9);
 		print_gfs2("0x%"PRIx64" / %"PRId64, ind->ii[pndx].block,
 			   ind->ii[pndx].block);
 		if (termlines) {
@@ -556,19 +554,21 @@ static int parse_rindex(struct gfs2_inode *dip, int print_rindex)
 			if(print_rindex)
 				gfs2_rindex_print(&ri);
 			else {
-				struct gfs2_buffer_head *tmp_bh;
+				char buf[sizeof(struct gfs2_rgrp)] = {0};
+				ssize_t ret;
 
-				tmp_bh = bread(&sbd, ri.ri_addr);
-				if (sbd.gfs1) {
+				ret = pread(sbd.device_fd, buf, sizeof(buf), ri.ri_addr * sbd.bsize);
+				if (ret != sizeof(buf)) {
+					perror("Failed to read resource group");
+				} else if (sbd.gfs1) {
 					struct gfs_rgrp rg1;
-					gfs_rgrp_in(&rg1, tmp_bh);
+					gfs_rgrp_in(&rg1, buf);
 					gfs_rgrp_print(&rg1);
 				} else {
 					struct gfs2_rgrp rg;
-					gfs2_rgrp_in(&rg, tmp_bh->b_data);
+					gfs2_rgrp_in(&rg, buf);
 					gfs2_rgrp_print(&rg);
 				}
-				brelse(tmp_bh);
 			}
 			last_entry_onscreen[dmode] = print_entry_ndx;
 		}
