@@ -8,7 +8,8 @@
 #endif
 
 #define ARRAY_SIZE(x) (sizeof(x)/sizeof(x[0]))
-#define SYM(x) { x, #x },
+#define SYM(x) { x, #x, 0 },
+#define PREFIX_SYM(p,x) { p##x, #p #x, sizeof #p - 1 },
 
 const struct lgfs2_symbolic lgfs2_metatypes[] = {
 SYM(GFS2_METATYPE_NONE)
@@ -70,7 +71,36 @@ SYM(GFS2_DIF_INHERIT_JDATA)
 const unsigned lgfs2_di_flag_size = ARRAY_SIZE(lgfs2_di_flags);
 
 const struct lgfs2_symbolic lgfs2_lh_flags[] = {
-SYM(GFS2_LOG_HEAD_UNMOUNT)
+PREFIX_SYM(GFS2_LOG_HEAD_, UNMOUNT)
+#ifdef GFS2_HAS_LH_V2
+PREFIX_SYM(GFS2_LOG_HEAD_FLUSH_, NORMAL)
+PREFIX_SYM(GFS2_LOG_HEAD_FLUSH_, SYNC)
+PREFIX_SYM(GFS2_LOG_HEAD_FLUSH_, SHUTDOWN)
+PREFIX_SYM(GFS2_LOG_HEAD_FLUSH_, FREEZE)
+PREFIX_SYM(GFS2_LOG_HEAD_, RECOVERY)
+PREFIX_SYM(GFS2_LOG_HEAD_, USERSPACE)
+
+/* Log flush callers */
+PREFIX_SYM(GFS2_LFC_, SHUTDOWN)
+PREFIX_SYM(GFS2_LFC_, JDATA_WPAGES)
+PREFIX_SYM(GFS2_LFC_, SET_FLAGS)
+PREFIX_SYM(GFS2_LFC_, AIL_EMPTY_GL)
+PREFIX_SYM(GFS2_LFC_, AIL_FLUSH)
+PREFIX_SYM(GFS2_LFC_, RGRP_GO_SYNC)
+PREFIX_SYM(GFS2_LFC_, INODE_GO_SYNC)
+PREFIX_SYM(GFS2_LFC_, INODE_GO_INVAL)
+PREFIX_SYM(GFS2_LFC_, FREEZE_GO_SYNC)
+PREFIX_SYM(GFS2_LFC_, KILL_SB)
+PREFIX_SYM(GFS2_LFC_, DO_SYNC)
+PREFIX_SYM(GFS2_LFC_, INPLACE_RESERVE)
+PREFIX_SYM(GFS2_LFC_, WRITE_INODE)
+PREFIX_SYM(GFS2_LFC_, MAKE_FS_RO)
+PREFIX_SYM(GFS2_LFC_, SYNC_FS)
+PREFIX_SYM(GFS2_LFC_, EVICT_INODE)
+PREFIX_SYM(GFS2_LFC_, TRANS_END)
+PREFIX_SYM(GFS2_LFC_, LOGD_JFLUSH_REQD)
+PREFIX_SYM(GFS2_LFC_, LOGD_AIL_FLUSH_REQD)
+#endif
 };
 
 const unsigned int lgfs2_lh_flag_size = ARRAY_SIZE(lgfs2_lh_flags);
@@ -358,7 +388,7 @@ MH(jd_header)
 static const struct lgfs2_metafield gfs2_log_header_fields[] = {
 MH(lh_header)
 F(lh_sequence)
-F(lh_flags)
+F(lh_flags, .flags = LGFS2_MFF_MASK, .symtab = lgfs2_lh_flags, .nsyms = ARRAY_SIZE(lgfs2_lh_flags))
 F(lh_tail)
 F(lh_blkno)
 F(lh_hash, .flags = LGFS2_MFF_CHECK)
@@ -381,7 +411,7 @@ F(lh_local_dinodes, .flags = LGFS2_MFF_FSBLOCKS)
 
 static const struct lgfs2_metafield gfs_log_header_fields[] = {
 MH(lh_header)
-F(lh_flags, .flags = LGFS2_MFF_MASK, .symtab = lgfs2_lh_flags, .nsyms = ARRAY_SIZE(lgfs2_lh_flags))
+F(lh_flags)
 RF(lh_pad)
 F(lh_first)
 F(lh_sequence)
@@ -530,6 +560,7 @@ const struct lgfs2_metadata lgfs2_metadata[] = {
 		.mh_type = GFS2_METATYPE_SB,
 		.mh_format = GFS2_FORMAT_SB,
 		.name = "gfs2_sb",
+		.display = "superblock",
 		.fields = gfs2_sb_fields,
 		.nfields = ARRAY_SIZE(gfs2_sb_fields),
 		.size = sizeof(struct gfs2_sb),
@@ -540,13 +571,15 @@ const struct lgfs2_metadata lgfs2_metadata[] = {
 		.mh_type = GFS2_METATYPE_SB,
 		.mh_format = GFS_FORMAT_SB,
 		.name = "gfs_sb",
+		.display = "superblock",
 		.fields = gfs_sb_fields,
 		.nfields = ARRAY_SIZE(gfs_sb_fields),
 		.size = sizeof(struct gfs_sb),
 	},
 	[LGFS2_MT_RINDEX] = {
 		.versions = LGFS2_MD_GFS1 | LGFS2_MD_GFS2,
-		.name = "rindex",
+		.name = "gfs2_rindex",
+		.display = "resource group index",
 		.fields = gfs2_rindex_fields,
 		.nfields = ARRAY_SIZE(gfs2_rindex_fields),
 		.size = sizeof(struct gfs2_rindex),
@@ -557,6 +590,7 @@ const struct lgfs2_metadata lgfs2_metadata[] = {
 		.mh_type = GFS2_METATYPE_RG,
 		.mh_format = GFS2_FORMAT_RG,
 		.name = "gfs2_rgrp",
+		.display = "resource group",
 		.fields = gfs2_rgrp_fields,
 		.nfields = ARRAY_SIZE(gfs2_rgrp_fields),
 		.size = sizeof(struct gfs2_rgrp),
@@ -567,6 +601,7 @@ const struct lgfs2_metadata lgfs2_metadata[] = {
 		.mh_type = GFS2_METATYPE_RG,
 		.mh_format = GFS2_FORMAT_RG,
 		.name = "gfs_rgrp",
+		.display = "resource group",
 		.fields = gfs_rgrp_fields,
 		.nfields = ARRAY_SIZE(gfs_rgrp_fields),
 		.size = sizeof(struct gfs_rgrp),
@@ -577,6 +612,7 @@ const struct lgfs2_metadata lgfs2_metadata[] = {
 		.mh_type = GFS2_METATYPE_RB,
 		.mh_format = GFS2_FORMAT_RB,
 		.name = "gfs2_rgrp_bitmap",
+		.display = "allocation bitmap",
 		.fields = gfs2_rgrp_bitmap_fields,
 		.nfields = ARRAY_SIZE(gfs2_rgrp_bitmap_fields),
 		.size = sizeof(struct gfs2_meta_header),
@@ -587,6 +623,7 @@ const struct lgfs2_metadata lgfs2_metadata[] = {
 		.mh_type = GFS2_METATYPE_DI,
 		.mh_format = GFS2_FORMAT_DI,
 		.name = "gfs2_dinode",
+		.display = "inode",
 		.fields = gfs2_dinode_fields,
 		.nfields = ARRAY_SIZE(gfs2_dinode_fields),
 		.size = sizeof(struct gfs2_dinode),
@@ -597,6 +634,7 @@ const struct lgfs2_metadata lgfs2_metadata[] = {
 		.mh_type = GFS2_METATYPE_DI,
 		.mh_format = GFS2_FORMAT_DI,
 		.name = "gfs_dinode",
+		.display = "inode",
 		.fields = gfs_dinode_fields,
 		.nfields = ARRAY_SIZE(gfs_dinode_fields),
 		.size = sizeof(struct gfs_dinode),
@@ -607,6 +645,7 @@ const struct lgfs2_metadata lgfs2_metadata[] = {
 		.mh_type = GFS2_METATYPE_IN,
 		.mh_format = GFS2_FORMAT_IN,
 		.name = "gfs2_indirect",
+		.display = "indirect",
 		.fields = gfs2_indirect_fields,
 		.nfields = ARRAY_SIZE(gfs2_indirect_fields),
 		.size = sizeof(struct gfs2_meta_header),
@@ -617,6 +656,7 @@ const struct lgfs2_metadata lgfs2_metadata[] = {
 		.mh_type = GFS2_METATYPE_IN,
 		.mh_format = GFS2_FORMAT_IN,
 		.name = "gfs_indirect",
+		.display = "indirect",
 		.fields = gfs_indirect_fields,
 		.nfields = ARRAY_SIZE(gfs_indirect_fields),
 		.size = sizeof(struct gfs_indirect),
@@ -627,6 +667,7 @@ const struct lgfs2_metadata lgfs2_metadata[] = {
 		.mh_type = GFS2_METATYPE_LF,
 		.mh_format = GFS2_FORMAT_LF,
 		.name = "gfs2_leaf",
+		.display = "leaf",
 		.fields = gfs2_leaf_fields,
 		.nfields = ARRAY_SIZE(gfs2_leaf_fields),
 		.size = sizeof(struct gfs2_leaf),
@@ -637,6 +678,7 @@ const struct lgfs2_metadata lgfs2_metadata[] = {
 		.mh_type = GFS2_METATYPE_JD,
 		.mh_format = GFS2_FORMAT_JD,
 		.name = "gfs2_jdata",
+		.display = "journal data",
 		.fields = gfs2_jdata_fields,
 		.nfields = ARRAY_SIZE(gfs2_jdata_fields),
 		.size = sizeof(struct gfs2_meta_header),
@@ -647,6 +689,7 @@ const struct lgfs2_metadata lgfs2_metadata[] = {
 		.mh_type = GFS2_METATYPE_LH,
 		.mh_format = GFS2_FORMAT_LH,
 		.name = "gfs2_log_header",
+		.display = "log header",
 		.fields = gfs2_log_header_fields,
 		.nfields = ARRAY_SIZE(gfs2_log_header_fields),
 		.size = sizeof(struct gfs2_log_header),
@@ -657,6 +700,7 @@ const struct lgfs2_metadata lgfs2_metadata[] = {
 		.mh_type = GFS2_METATYPE_LH,
 		.mh_format = GFS2_FORMAT_LH,
 		.name = "gfs_log_header",
+		.display = "log header",
 		.fields = gfs_log_header_fields,
 		.nfields = ARRAY_SIZE(gfs_log_header_fields),
 		.size = sizeof(struct gfs_log_header),
@@ -667,6 +711,7 @@ const struct lgfs2_metadata lgfs2_metadata[] = {
 		.mh_type = GFS2_METATYPE_LD,
 		.mh_format = GFS2_FORMAT_LD,
 		.name = "gfs2_log_desc",
+		.display = "log descriptor",
 		.fields = gfs2_log_desc_fields,
 		.nfields = ARRAY_SIZE(gfs2_log_desc_fields),
 		.size = sizeof(struct gfs2_log_descriptor),
@@ -677,6 +722,7 @@ const struct lgfs2_metadata lgfs2_metadata[] = {
 		.mh_type = GFS2_METATYPE_LD,
 		.mh_format = GFS2_FORMAT_LD,
 		.name = "gfs_log_desc",
+		.display = "log descriptor",
 		.fields = gfs_log_desc_fields,
 		.nfields = ARRAY_SIZE(gfs_log_desc_fields),
 		.size = sizeof(struct gfs_log_descriptor),
@@ -687,6 +733,7 @@ const struct lgfs2_metadata lgfs2_metadata[] = {
 		.mh_type = GFS2_METATYPE_LB,
 		.mh_format = GFS2_FORMAT_LB,
 		.name = "gfs2_log_block",
+		.display = "log",
 		.fields = gfs2_log_block_fields,
 		.nfields = ARRAY_SIZE(gfs2_log_block_fields),
 		.size = sizeof(struct gfs2_meta_header),
@@ -697,6 +744,7 @@ const struct lgfs2_metadata lgfs2_metadata[] = {
 		.mh_type = GFS2_METATYPE_EA,
 		.mh_format = GFS2_FORMAT_EA,
 		.name = "gfs2_ea_attr",
+		.display = "extended attribute",
 		.fields = gfs2_ea_attr_fields,
 		.nfields = ARRAY_SIZE(gfs2_ea_attr_fields),
 		.size = sizeof(struct gfs2_meta_header),
@@ -707,6 +755,7 @@ const struct lgfs2_metadata lgfs2_metadata[] = {
 		.mh_type = GFS2_METATYPE_ED,
 		.mh_format = GFS2_FORMAT_ED,
 		.name = "gfs2_ea_data",
+		.display = "extended attribute data",
 		.fields = gfs2_ea_data_fields,
 		.nfields = ARRAY_SIZE(gfs2_ea_data_fields),
 		.size = sizeof(struct gfs2_meta_header),
@@ -714,6 +763,7 @@ const struct lgfs2_metadata lgfs2_metadata[] = {
 	[LGFS2_MT_GFS2_QUOTA_CHANGE] = {
 		.versions = LGFS2_MD_GFS2,
 		.name = "gfs2_quota_change",
+		.display = "quota change",
 		.fields = gfs2_quota_change_fields,
 		.nfields = ARRAY_SIZE(gfs2_quota_change_fields),
 		.size = sizeof(struct gfs2_quota_change),
@@ -721,6 +771,7 @@ const struct lgfs2_metadata lgfs2_metadata[] = {
 	[LGFS2_MT_DIRENT] = {
 		.versions = LGFS2_MD_GFS1 | LGFS2_MD_GFS2,
 		.name = "gfs2_dirent",
+		.display = "directory entry",
 		.fields = gfs2_dirent_fields,
 		.nfields = ARRAY_SIZE(gfs2_dirent_fields),
 		.size = sizeof(struct gfs2_dirent),
@@ -728,6 +779,7 @@ const struct lgfs2_metadata lgfs2_metadata[] = {
 	[LGFS2_MT_EA_HEADER] = {
 		.versions = LGFS2_MD_GFS1 | LGFS2_MD_GFS2,
 		.name = "gfs2_ea_header",
+		.display = "extended attribute header",
 		.fields = gfs2_ea_header_fields,
 		.nfields = ARRAY_SIZE(gfs2_ea_header_fields),
 		.size = sizeof(struct gfs2_ea_header),
@@ -735,6 +787,7 @@ const struct lgfs2_metadata lgfs2_metadata[] = {
 	[LGFS2_MT_GFS2_INUM_RANGE] = {
 		.versions = LGFS2_MD_GFS2,
 		.name = "gfs2_inum_range",
+		.display = "inum range",
 		.fields = gfs2_inum_range_fields,
 		.nfields = ARRAY_SIZE(gfs2_inum_range_fields),
 		.size = sizeof(struct gfs2_inum_range),
@@ -742,6 +795,7 @@ const struct lgfs2_metadata lgfs2_metadata[] = {
 	[LGFS2_MT_STATFS_CHANGE] = {
 		.versions = LGFS2_MD_GFS1 | LGFS2_MD_GFS2,
 		.name = "gfs2_statfs_change",
+		.display = "statfs change",
 		.fields = gfs2_statfs_change_fields,
 		.nfields = ARRAY_SIZE(gfs2_statfs_change_fields),
 		.size = sizeof(struct gfs2_statfs_change),
@@ -749,6 +803,7 @@ const struct lgfs2_metadata lgfs2_metadata[] = {
 	[LGFS2_MT_GFS_JINDEX] = {
 		.versions = LGFS2_MD_GFS1,
 		.name = "gfs_jindex",
+		.display = "journal index",
 		.fields = gfs_jindex_fields,
 		.nfields = ARRAY_SIZE(gfs_jindex_fields),
 		.size = sizeof(struct gfs_jindex),
@@ -756,6 +811,7 @@ const struct lgfs2_metadata lgfs2_metadata[] = {
 	[LGFS2_MT_GFS_BLOCK_TAG] = {
 		.versions = LGFS2_MD_GFS1,
 		.name = "gfs_block_tag",
+		.display = "block tag",
 		.fields = gfs_block_tag_fields,
 		.nfields = ARRAY_SIZE(gfs_block_tag_fields),
 		.size = sizeof(struct gfs_block_tag),
@@ -763,10 +819,12 @@ const struct lgfs2_metadata lgfs2_metadata[] = {
 	[LGFS2_MT_DATA] = {
 		.versions = LGFS2_MD_GFS1 | LGFS2_MD_GFS2,
 		.name = "data",
+		.display = "data",
 	},
 	[LGFS2_MT_FREE] = {
 		.versions = LGFS2_MD_GFS1 | LGFS2_MD_GFS2,
 		.name = "free",
+		.display = "free",
 	},
 };
 
@@ -916,23 +974,58 @@ int lgfs2_field_str(char *str, const size_t size, const char *blk, const struct
 	} else if (field->flags & LGFS2_MFF_STRING) {
 		snprintf(str, size, "%s", fieldp);
 	} else {
+		uint64_t value;
+
 		switch(field->length) {
 		case sizeof(uint8_t):
-			snprintf(str, size, hex? "%"PRIx8 : "%"PRIu8, *(uint8_t *)fieldp);
+			value = *(uint8_t *)fieldp;
 			break;
 		case sizeof(uint16_t):
-			snprintf(str, size, hex? "%"PRIx16 : "%"PRIu16, be16_to_cpu(*(uint16_t *)fieldp));
+			value = be16_to_cpu(*(uint16_t *)fieldp);
 			break;
 		case sizeof(uint32_t):
-			snprintf(str, size, hex? "%"PRIx32 : "%"PRIu32, be32_to_cpu(*(uint32_t *)fieldp));
+			value = be32_to_cpu(*(uint32_t *)fieldp);
 			break;
 		case sizeof(uint64_t):
-			snprintf(str, size, hex? "%"PRIx64 : "%"PRIu64, be64_to_cpu(*(uint64_t *)fieldp));
+			value = be64_to_cpu(*(uint64_t *)fieldp);
 			break;
 		default:
-			break;
+			*str = 0;
+			goto out;
 		}
+
+		if (hex) {
+			snprintf(str, size, "%"PRIx64, value);
+			goto out;
+		}
+		if (field->flags & LGFS2_MFF_MASK) {
+			char *s = str, *end = str + size;
+			unsigned n;
+
+			for (n = 0; n < field->nsyms; n++) {
+				const struct lgfs2_symbolic *sym = &field->symtab[n];
+
+				if (!(value & sym->key))
+					continue;
+				if (s != str && s < end)
+					*s++ = '/';
+				if (s < end) {
+					const char *v = sym->value + sym->prefix;
+					size_t len = strlen(v);
+
+					if (len > end - s)
+						len = end - s;
+					memcpy(s, v, len);
+					s += len;
+				}
+			}
+			if (s < end)
+				*s = '\0';
+			goto out;
+		}
+		snprintf(str, size, "%"PRIu64, value);
 	}
+out:
 	str[size - 1] = '\0';
 	return 0;
 }
@@ -940,6 +1033,7 @@ int lgfs2_field_str(char *str, const size_t size, const char *blk, const struct
 int lgfs2_field_assign(char *blk, const struct lgfs2_metafield *field, const void *val)
 {
 	char *fieldp = blk + field->offset;
+	uint64_t num = *(uint64_t *)val;
 
 	if (field->flags & LGFS2_MFF_UUID) {
 		memcpy(fieldp, val, 16);
@@ -959,16 +1053,16 @@ int lgfs2_field_assign(char *blk, const struct lgfs2_metafield *field, const voi
 
 	switch(field->length) {
 	case sizeof(uint8_t):
-		*fieldp = *(uint8_t *)val;
+		*fieldp = (uint8_t)num;
 		return 0;
 	case sizeof(uint16_t):
-		*(uint16_t *)fieldp = cpu_to_be16(*(uint16_t *)val);
+		*(uint16_t *)fieldp = cpu_to_be16((uint16_t)num);
 		return 0;
 	case sizeof(uint32_t):
-		*(uint32_t *)fieldp = cpu_to_be32(*(uint32_t *)val);
+		*(uint32_t *)fieldp = cpu_to_be32((uint32_t)num);
 		return 0;
 	case sizeof(uint64_t):
-		*(uint64_t *)fieldp = cpu_to_be64(*(uint64_t *)val);
+		*(uint64_t *)fieldp = cpu_to_be64((uint64_t)num);
 		return 0;
 	default:
 		/* Will never happen */
