@@ -81,6 +81,7 @@ static void print_ext_opts(void)
 		"swidth=N",  _("Specify the stripe width of the device, overriding probed values"),
 		"sunit=N", _("Specify the stripe unit of the device, overriding probed values"),
 		"align=[0|1]", _("Disable or enable alignment of resource groups"),
+		"format=N", _("Specify the format version number"),
 		NULL, NULL
 	};
 	printf(_("Extended options:\n"));
@@ -118,8 +119,9 @@ struct mkfs_opts {
 	unsigned rgsize;
 	unsigned long sunit;
 	unsigned long swidth;
+	unsigned format;
 	uint64_t fssize;
-	uint32_t journals;
+	int journals;
 	const char *lockproto;
 	const char *locktable;
 	struct mkfs_dev dev;
@@ -137,12 +139,13 @@ struct mkfs_opts {
 	unsigned got_locktable:1;
 	unsigned got_device:1;
 	unsigned got_topol:1;
+	unsigned got_format:1;
 
 	unsigned override:1;
 	unsigned quiet:1;
 	unsigned debug:1;
 	unsigned confirm:1;
-	unsigned align:1;
+	unsigned align;
 };
 
 static void opts_init(struct mkfs_opts *opts)
@@ -158,6 +161,7 @@ static void opts_init(struct mkfs_opts *opts)
 	opts->locktable = "";
 	opts->confirm = 1;
 	opts->align = 1;
+	opts->format = GFS2_FORMAT_FS;
 }
 
 struct gfs2_inum *mkfs_journals = NULL;
@@ -208,7 +212,7 @@ static long long cvtnum(unsigned int blocksize, unsigned int sectorsize, const c
                 if (blocksize)
                         return i * blocksize;
                 fprintf(stderr, _("Block size not available yet.\n"));
-		exit(1);
+		return -1LL;
         }
         if (*sp == 's' && sp[1] == '\0') {
                 if (sectorsize)
@@ -230,31 +234,36 @@ static long long cvtnum(unsigned int blocksize, unsigned int sectorsize, const c
         return -1LL;
 }
 
-static unsigned long parse_ulong(struct mkfs_opts *opts, const char *key, const char *val)
+static int parse_ulong(struct mkfs_opts *opts, const char *key, const char *val, unsigned long *pn)
 {
 	long long l;
 	if (val == NULL || *val == '\0') {
 		fprintf(stderr, _("Missing argument to '%s'\n"), key);
-		exit(-1);
+		return -1;
 	}
 	l = cvtnum(opts->bsize, 0, val);
 	if (l > ULONG_MAX || l < 0) {
 		fprintf(stderr, _("Value of '%s' is invalid\n"), key);
-		exit(-1);
+		return -1;
 	}
-	return (unsigned long)l;
+	*pn = (unsigned long)l;
+	return 0;
 }
 
-static unsigned parse_bool(struct mkfs_opts *opts, const char *key, const char *val)
+static int parse_bool(struct mkfs_opts *opts, const char *key, const char *val, unsigned *pbool)
 {
 	if (strnlen(val, 2) == 1) {
-		if (*val == '0')
+		if (*val == '0') {
+			*pbool = 0;
 			return 0;
-		if (*val == '1')
-			return 1;
+		}
+		if (*val == '1') {
+			*pbool = 1;
+			return 0;
+		}
 	}
 	fprintf(stderr, _("Option '%s' must be either 1 or 0\n"), key);
-	exit(-1);
+	return -1;
 }
 
 static int parse_topology(struct mkfs_opts *opts, char *str)
@@ -274,7 +283,8 @@ static int parse_topology(struct mkfs_opts *opts, char *str)
 			fprintf(stderr, "Too many topology values.\n");
 			return 1;
 		}
-		*topol[i] = parse_ulong(opts, "test_topology", opt);
+		if (parse_ulong(opts, "test_topology", opt, topol[i]))
+			return 1;
 		i++;
 	}
 	if (i < 5) {
@@ -284,7 +294,23 @@ static int parse_topology(struct mkfs_opts *opts, char *str)
 	return 0;
 }
 
-static void opt_parse_extended(char *str, struct mkfs_opts *opts)
+static int parse_format(struct mkfs_opts *opts, char *str)
+{
+	unsigned long ln;
+
+	if (parse_ulong(opts, "format", str, &ln) != 0)
+		return -1;
+
+	if (ln < LGFS2_FS_FORMAT_MIN || ln > LGFS2_FS_FORMAT_MAX) {
+		fprintf(stderr, _("Invalid filesystem format: %s\n"), str);
+		return -1;
+	}
+	opts->format = ln;
+	opts->got_format = 1;
+	return 0;
+}
+
+static int opt_parse_extended(char *str, struct mkfs_opts *opts)
 {
 	char *opt;
 	while ((opt = strsep(&str, ",")) != NULL) {
@@ -292,34 +318,42 @@ static void opt_parse_extended(char *str, struct mkfs_opts *opts)
 		char *val = strsep(&opt, "=");
 		if (key == NULL || *key == '\0') {
 			fprintf(stderr, _("Missing argument to '-o' option\n"));
-			exit(-1);
+			return -1;
 		}
 		if (strcmp("sunit", key) == 0) {
-			opts->sunit = parse_ulong(opts, "sunit", val);
+			if (parse_ulong(opts, "sunit", val, &opts->sunit) != 0)
+				return -1;
 			opts->got_sunit = 1;
 		} else if (strcmp("swidth", key) == 0) {
-			opts->swidth = parse_ulong(opts, "swidth", val);
+			if (parse_ulong(opts, "swidth", val, &opts->swidth) != 0)
+				return -1;
 			opts->got_swidth = 1;
 		} else if (strcmp("align", key) == 0) {
-			opts->align = parse_bool(opts, "align", val);
+			if (parse_bool(opts, "align", val, &opts->align) != 0)
+				return -1;
 		} else if (strcmp("test_topology", key) == 0) {
 			if (parse_topology(opts, val) != 0)
-				exit(-1);
+				return -1;
 			opts->got_topol = (opts->dev.logical_sector_size != 0 &&
 	                                   opts->dev.physical_sector_size != 0);
+		} else if (strcmp("format", key) == 0) {
+			if (parse_format(opts, val) != 0)
+				return -1;
 		} else if (strcmp("help", key) == 0) {
 			print_ext_opts();
-			exit(0);
+			return 1;
 		} else {
 			fprintf(stderr, _("Invalid extended option (specified with -o): '%s'\n"), key);
 			print_ext_opts();
-			exit(-1);
+			return -1;
 		}
 	}
+	return 0;
 }
 
-static void opts_get(int argc, char *argv[], struct mkfs_opts *opts)
+static int opts_get(int argc, char *argv[], struct mkfs_opts *opts)
 {
+	int ret;
 	int c;
 	while (1) {
 		c = getopt(argc, argv, "-b:c:DhJ:j:KOo:p:qr:t:V");
@@ -341,7 +375,7 @@ static void opts_get(int argc, char *argv[], struct mkfs_opts *opts)
 			break;
 		case 'h':
 			print_usage(argv[0]);
-			exit(0);
+			return 1;
 		case 'J':
 			opts->jsize = atoi(optarg);
 			opts->got_jsize = 1;
@@ -372,18 +406,20 @@ static void opts_get(int argc, char *argv[], struct mkfs_opts *opts)
 			opts->got_rgsize = 1;
 			break;
 		case 'o':
-			opt_parse_extended(optarg, opts);
+			ret = opt_parse_extended(optarg, opts);
+			if (ret != 0)
+				return ret;
 			break;
 		case 'V':
 			printf("mkfs.gfs2 %s (built %s %s)\n", VERSION,
 			       __DATE__, __TIME__);
 			printf(REDHAT_COPYRIGHT "\n");
-			exit(EXIT_SUCCESS);
+			return 1;
 			break;
 		case ':':
 		case '?':
 			fprintf(stderr, _("Please use '-h' for help.\n"));
-			exit(EXIT_FAILURE);
+			return -1;
 			break;
 		case 1:
 			if (strcmp(optarg, "gfs2") == 0)
@@ -394,24 +430,27 @@ static void opts_get(int argc, char *argv[], struct mkfs_opts *opts)
 			} else if (!opts->got_fssize && isdigit(optarg[0])) {
 				opts->fssize = atol(optarg);
 				opts->got_fssize = 1;
-			} else
-				die( _("More than one device specified (try -h for help)\n"));
+			} else {
+				fprintf(stderr, _("More than one device specified (try -h for help)\n"));
+				return -1;
+			}
 			break;
 		default:
-			die( _("Invalid option: %c\n"), c);
-			break;
+			fprintf(stderr, _("Invalid option: %c\n"), c);
+			return -1;
 		};
 	}
+	return 0;
 }
 
 /**
- * test_locking - Make sure the GFS2 is set up to use the right lock protocol
+ * Make sure the GFS2 is set up to use the right lock protocol
  * @lockproto: the lock protocol to mount
  * @locktable: the locktable name
  *
  */
 
-static void test_locking(struct mkfs_opts *opts)
+static int test_locking(struct mkfs_opts *opts)
 {
 	const char *c;
 	/* Translators: A lock table is a string identifying a gfs2 file system
@@ -420,45 +459,61 @@ static void test_locking(struct mkfs_opts *opts)
 	int table_required = (strcmp(opts->lockproto, "lock_gulm") == 0)
 	                  || (strcmp(opts->lockproto, "lock_dlm") == 0);
 
-	if ((strcmp(opts->lockproto, "lock_nolock") != 0) && !table_required)
-		die( _("Invalid lock protocol: %s\n"), opts->lockproto);
-
+	if ((strcmp(opts->lockproto, "lock_nolock") != 0) && !table_required) {
+		fprintf(stderr, _("Invalid lock protocol: %s\n"), opts->lockproto);
+		return -1;
+	}
 	/* When lock_*lm is given as the lock protocol, require a lock table too */
 	if (!opts->got_locktable) {
 		if (table_required) {
 			fprintf(stderr, _("No lock table specified.\n"));
-			exit(-1);
+			return -1;
 		}
-		return;
+		return 0;
 	}
 	/* User gave a lock table option, validate it */
 	if (*opts->locktable == '\0') {
 		fprintf(stderr, _("Lock table is empty.\n"));
-		exit(-1);
+		return -1;
 	}
 	for (c = opts->locktable; *c; c++) {
-		if (!isalnum(*c) && (*c != '-') && (*c != '_') && (*c != ':'))
-			die("%s %s '%c'\n", errprefix, _("invalid character"), *c);
+		if (!isalnum(*c) && (*c != '-') && (*c != '_') && (*c != ':')) {
+			fprintf(stderr, "%s %s '%c'\n", errprefix, _("invalid character"), *c);
+			return -1;
+		}
 	}
 	c = strstr(opts->locktable, ":");
-	if (!c)
-		die("%s %s\n", errprefix, _("missing colon"));
+	if (!c) {
+		fprintf(stderr, "%s %s\n", errprefix, _("missing colon"));
+		return -1;
+	}
 
-	if (c == opts->locktable)
-		die("%s %s\n", errprefix, _("cluster name is missing"));
-	if (c - opts->locktable > 32)
-		die("%s %s\n", errprefix, _("cluster name is too long"));
+	if (c == opts->locktable) {
+		fprintf(stderr, "%s %s\n", errprefix, _("cluster name is missing"));
+		return -1;
+	}
+	if (c - opts->locktable > 32) {
+		fprintf(stderr, "%s %s\n", errprefix, _("cluster name is too long"));
+		return -1;
+	}
 
 	c++;
-	if (strstr(c, ":"))
-		die("%s %s\n", errprefix, _("contains more than one colon"));
-	if (!strlen(c))
-		die("%s %s\n", errprefix, _("file system name is missing"));
-	if (strlen(c) > 30)
-		die("%s %s\n", errprefix, _("file system name is too long"));
+	if (strstr(c, ":")) {
+		fprintf(stderr, "%s %s\n", errprefix, _("contains more than one colon"));
+		return -1;
+	}
+	if (!strlen(c)) {
+		fprintf(stderr, "%s %s\n", errprefix, _("file system name is missing"));
+		return -1;
+	}
+	if (strlen(c) > 30) {
+		fprintf(stderr, "%s %s\n", errprefix, _("file system name is too long"));
+		return -1;
+	}
+	return 0;
 }
 
-static void are_you_sure(void)
+static int are_you_sure(void)
 {
 	while (1) {
 		char *line = NULL;
@@ -473,19 +528,24 @@ static void are_you_sure(void)
 		   `locale -k noexpr` */
 		printf( _("Are you sure you want to proceed? [y/n] "));
 		ret = getline(&line, &len, stdin);
+		if (ret < 0) {
+			printf("\n");
+			free(line);
+			return 0;
+		}
 		res = rpmatch(line);
 		free(line);
-		if (ret <= 0)
+		if (ret == 0)
 			continue;
 		if (res == 1) /* Yes */
-			return;
+			return 1;
 		if (res == 0) /* No */
-			exit(1);
+			return 0;
 		/* Unrecognized input; go again. */
-	};
+	}
 }
 
-static unsigned choose_blocksize(struct mkfs_opts *opts)
+static int choose_blocksize(struct mkfs_opts *opts, unsigned *pbsize)
 {
 	unsigned int x;
 	unsigned int bsize = opts->bsize;
@@ -512,57 +572,71 @@ static unsigned choose_blocksize(struct mkfs_opts *opts)
 		         dev->physical_sector_size >= GFS2_DEFAULT_BSIZE)
 			bsize = dev->physical_sector_size;
 	}
-
 	/* Block sizes must be a power of two from 512 to 65536 */
 	for (x = 512; x; x <<= 1)
 		if (x == bsize)
 			break;
 
-	if (!x || bsize > getpagesize())
-		die( _("Block size must be a power of two between 512 and %d\n"),
-		       getpagesize());
-
+	if (!x || bsize > getpagesize()) {
+		fprintf(stderr, _("Block size must be a power of two between 512 and %d\n"),
+		        getpagesize());
+		return -1;
+	}
 	if (bsize < dev->logical_sector_size) {
-		die( _("Error: Block size %d is less than minimum logical "
+		fprintf(stderr, _("Error: Block size %d is less than minimum logical "
 		       "block size (%lu).\n"), bsize, dev->logical_sector_size);
+		return -1;
 	}
-
 	if (bsize < dev->physical_sector_size) {
 		printf( _("Warning: Block size %d is inefficient because it "
 			  "is less than the physical block size (%lu).\n"),
 			  bsize, dev->physical_sector_size);
 		opts->confirm = 1;
 	}
-	return bsize;
+	*pbsize = bsize;
+	return 0;
 }
 
-static void opts_check(struct mkfs_opts *opts)
+static int opts_check(struct mkfs_opts *opts)
 {
 	if (!opts->got_device) {
 		fprintf(stderr, _("No device specified. Use -h for help\n"));
-		exit(1);
+		return -1;
 	}
 
-	test_locking(opts);
+	if (test_locking(opts) != 0)
+		return -1;
 
-	if (GFS2_MIN_RGSIZE > opts->rgsize || opts->rgsize > GFS2_MAX_RGSIZE)
+	if (GFS2_MIN_RGSIZE > opts->rgsize || opts->rgsize > GFS2_MAX_RGSIZE) {
 		/* Translators: gfs2 file systems are split into equal sized chunks called
 		   resource groups. We're checking that the user gave a valid size for them. */
-		die( _("bad resource group size\n"));
+		fprintf(stderr, _("Bad resource group size\n"));
+		return -1;
+	}
 
-	if (!opts->journals)
-		die( _("no journals specified\n"));
+	if (!opts->journals) {
+		fprintf(stderr, _("No journals specified\n"));
+		return -1;
+	}
+	if (opts->journals < 0) {
+		fprintf(stderr, _("Number of journals cannot be negative: %d\n"), opts->journals);
+		return -1;
+	}
+	if (opts->jsize < GFS2_MIN_JSIZE || opts->jsize > GFS2_MAX_JSIZE) {
+		fprintf(stderr, _("Bad journal size\n"));
+		return -1;
+	}
 
-	if (opts->jsize < GFS2_MIN_JSIZE || opts->jsize > GFS2_MAX_JSIZE)
-		die( _("bad journal size\n"));
-
-	if (!opts->qcsize || opts->qcsize > 64)
-		die( _("bad quota change size\n"));
+	if (!opts->qcsize || opts->qcsize > 64) {
+		fprintf(stderr, _("Bad quota change size\n"));
+		return -1;
+	}
 
 	if ((opts->got_sunit && !opts->got_swidth) || (!opts->got_sunit && opts->got_swidth)) {
 		fprintf(stderr, _("Stripe unit and stripe width must be specified together\n"));
-		exit(1);
+		return -1;
 	}
+	return 0;
 }
 
 static void print_results(struct gfs2_sb *sb, struct mkfs_opts *opts, uint64_t rgrps, uint64_t fssize)
@@ -591,20 +665,20 @@ static void print_results(struct gfs2_sb *sb, struct mkfs_opts *opts, uint64_t r
 #endif
 }
 
-static void warn_of_destruction(const char *path)
+static int warn_of_destruction(const char *path)
 {
 	struct stat lnkstat;
 	char *abspath = NULL;
 
 	if (lstat(path, &lnkstat) == -1) {
 		perror(_("Failed to lstat the device"));
-		exit(EXIT_FAILURE);
+		return -1;
 	}
 	if (S_ISLNK(lnkstat.st_mode)) {
 		abspath = canonicalize_file_name(path);
 		if (abspath == NULL) {
 			perror(_("Could not find the absolute path of the device"));
-			exit(EXIT_FAILURE);
+			return -1;
 		}
 		/* Translators: Example: "/dev/vg/lv is a symbolic link to /dev/dm-2" */
 		printf( _("%s is a symbolic link to %s\n"), path, abspath);
@@ -612,6 +686,7 @@ static void warn_of_destruction(const char *path)
 	}
 	printf(_("This will destroy any data on %s\n"), path);
 	free(abspath);
+	return 0;
 }
 
 static int zero_gap(struct gfs2_sbd *sdp, uint64_t addr, size_t blocks)
@@ -660,11 +735,11 @@ static lgfs2_rgrps_t rgs_init(struct mkfs_opts *opts, struct gfs2_sbd *sdp)
 		if ((opts->sunit % sdp->bsize) != 0) {
 			fprintf(stderr, _("Stripe unit (%lu) must be a multiple of block size (%u)\n"),
 			        opts->sunit, sdp->bsize);
-			exit(1);
+			return NULL;
 		} else if ((opts->swidth % opts->sunit) != 0) {
 			fprintf(stderr, _("Stripe width (%lu) must be a multiple of stripe unit (%lu)\n"),
 			        opts->swidth, opts->sunit);
-			exit(1);
+			return NULL;
 		} else {
 			al_base = opts->swidth / sdp->bsize;
 			al_off = opts->sunit / sdp->bsize;
@@ -686,7 +761,7 @@ static lgfs2_rgrps_t rgs_init(struct mkfs_opts *opts, struct gfs2_sbd *sdp)
 	rgs = lgfs2_rgrps_init(sdp, al_base, al_off);
 	if (rgs == NULL) {
 		perror(_("Could not initialise resource groups"));
-		exit(-1);
+		return NULL;
 	}
 
 	if (opts->debug) {
@@ -859,11 +934,13 @@ static int place_rgrps(struct gfs2_sbd *sdp, lgfs2_rgrps_t rgs, uint64_t *rgaddr
 			fprintf(stderr, _("Failed to build resource groups\n"));
 			return result;
 		}
-
 		gfs2_progress_update(&progress, (sdp->rgrps));
 	}
+	if (lgfs2_rgrps_write_final(sdp->device_fd, rgs) != 0) {
+		perror(_("Failed to write final resource group"));
+		return 0;
+	}
 	gfs2_progress_close(&progress, _("Done\n"));
-	posix_fadvise(sdp->device_fd, 0, sdp->fssize * sdp->bsize, POSIX_FADV_DONTNEED);
 
 	return 0;
 }
@@ -898,7 +975,7 @@ static int default_journal_size(unsigned bsize, uint64_t num_blocks)
 	return 262144;                          /*   1 GB */
 }
 
-static void sbd_init(struct gfs2_sbd *sdp, struct mkfs_opts *opts, unsigned bsize)
+static int sbd_init(struct gfs2_sbd *sdp, struct mkfs_opts *opts, unsigned bsize)
 {
 	memset(sdp, 0, sizeof(struct gfs2_sbd));
 	sdp->time = time(NULL);
@@ -912,15 +989,16 @@ static void sbd_init(struct gfs2_sbd *sdp, struct mkfs_opts *opts, unsigned bsiz
 
 	if (compute_constants(sdp)) {
 		perror(_("Failed to compute file system constants"));
-		exit(1);
+		return -1;
 	}
 	sdp->device.length = opts->dev.size / sdp->bsize;
 	if (opts->got_fssize) {
 		if (opts->fssize > sdp->device.length) {
 			fprintf(stderr, _("Specified size is bigger than the device."));
-			die("%s %.2f %s (%"PRIu64" %s)\n", _("Device size:"),
+			fprintf(stderr, "%s %.2f %s (%"PRIu64" %s)\n", _("Device size:"),
 			       opts->dev.size / ((float)(1 << 30)), _("GB"),
 			       opts->dev.size / sdp->bsize, _("blocks"));
+			return -1;
 		}
 		sdp->device.length = opts->fssize;
 	}
@@ -933,7 +1011,7 @@ static void sbd_init(struct gfs2_sbd *sdp, struct mkfs_opts *opts, unsigned bsiz
 
 		if (default_jsize < 0) {
 			fprintf(stderr, _("gfs2 will not fit on this device.\n"));
-			exit(1);
+			return -1;
 		}
 		jsize_mb = (default_jsize * sdp->bsize) >> 20;
 		if (jsize_mb < GFS2_MIN_JSIZE)
@@ -947,9 +1025,10 @@ static void sbd_init(struct gfs2_sbd *sdp, struct mkfs_opts *opts, unsigned bsiz
 		if (max_jsize >= GFS2_MIN_JSIZE)
 			fprintf(stderr, _("Maximum size for %u journals on this device is %uMB.\n"),
 			        opts->journals, max_jsize);
-		exit(1);
+		return -1;
 	}
 	sdp->jsize = opts->jsize;
+	return 0;
 }
 
 static int probe_contents(struct mkfs_dev *dev)
@@ -958,13 +1037,13 @@ static int probe_contents(struct mkfs_dev *dev)
 	const char *contents;
 	blkid_probe pr = blkid_new_probe();
 	if (pr == NULL || blkid_probe_set_device(pr, dev->fd, 0, 0) != 0
-	               || blkid_probe_enable_superblocks(pr, TRUE) != 0
-	               || blkid_probe_enable_partitions(pr, TRUE) != 0) {
+	               || blkid_probe_enable_superblocks(pr, 1) != 0
+	               || blkid_probe_enable_partitions(pr, 1) != 0) {
 		fprintf(stderr, _("Failed to create probe\n"));
 		return -1;
 	}
 
-	if (!S_ISREG(dev->stat.st_mode) && blkid_probe_enable_topology(pr, TRUE) != 0) {
+	if (!S_ISREG(dev->stat.st_mode) && blkid_probe_enable_topology(pr, 1) != 0) {
 		fprintf(stderr, _("Failed to create probe\n"));
 		return -1;
 	}
@@ -1001,25 +1080,23 @@ static int probe_contents(struct mkfs_dev *dev)
 	return 0;
 }
 
-static void open_dev(struct mkfs_dev *dev, int withprobe)
+static int open_dev(struct mkfs_dev *dev, int withprobe)
 {
 	int error;
 
 	dev->fd = open(dev->path, O_RDWR|O_CLOEXEC|O_EXCL);
 	if (dev->fd < 0) {
 		perror(dev->path);
-		exit(1);
+		return 1;
 	}
 
 	/* Freshen up the cache */
 	posix_fadvise(dev->fd, 0, 0, POSIX_FADV_DONTNEED);
-	/* Turn off readahead, we're just writing new blocks */
-	posix_fadvise(dev->fd, 0, 0, POSIX_FADV_RANDOM);
 
 	error = fstat(dev->fd, &dev->stat);
 	if (error < 0) {
 		perror(dev->path);
-		exit(1);
+		return 1;
 	}
 
 	if (S_ISREG(dev->stat.st_mode)) {
@@ -1028,16 +1105,18 @@ static void open_dev(struct mkfs_dev *dev, int withprobe)
 		dev->size = lseek(dev->fd, 0, SEEK_END);
 		if (dev->size < 1) {
 			fprintf(stderr, _("Device '%s' is too small\n"), dev->path);
-			exit(1);
+			return 1;
 		}
 	} else {
 		fprintf(stderr, _("'%s' is not a block device or regular file\n"), dev->path);
-		exit(1);
+		return 1;
 	}
 	if (withprobe && (probe_contents(dev) != 0))
-		exit(1);
+		return 1;
+	return 0;
 }
 
+#ifndef UNITTESTS
 int main(int argc, char *argv[])
 {
 	struct gfs2_sbd sbd;
@@ -1053,18 +1132,29 @@ int main(int argc, char *argv[])
 	srandom(time(NULL) ^ getpid());
 
 	opts_init(&opts);
-	opts_get(argc, argv, &opts);
-	opts_check(&opts);
+	error = opts_get(argc, argv, &opts);
+	if (error == 1)
+		exit(0);
+	if (error != 0)
+		exit(-1);
+	error = opts_check(&opts);
+	if (error != 0)
+		exit(error);
 
-	open_dev(&opts.dev, !opts.got_topol);
-	bsize = choose_blocksize(&opts);
+	error = open_dev(&opts.dev, !opts.got_topol);
+	if (error != 0)
+		exit(error);
+	error = choose_blocksize(&opts, &bsize);
+	if (error != 0)
+		exit(-1);
 
 	if (S_ISREG(opts.dev.stat.st_mode)) {
 		opts.got_bsize = 1; /* Use default block size for regular files */
 	}
 
-	sbd_init(&sbd, &opts, bsize);
-	lgfs2_sb_init(&sb, bsize);
+	if (sbd_init(&sbd, &opts, bsize) != 0)
+		exit(-1);
+	lgfs2_sb_init(&sb, bsize, opts.format);
 	if (opts.debug) {
 		printf(_("File system options:\n"));
 		printf("  bsize = %u\n", sbd.bsize);
@@ -1077,12 +1167,17 @@ int main(int argc, char *argv[])
 		printf("  fssize = %"PRIu64"\n", opts.fssize);
 		printf("  sunit = %lu\n", opts.sunit);
 		printf("  swidth = %lu\n", opts.swidth);
+		printf("  format = %u\n", opts.format);
 	}
 	rgs = rgs_init(&opts, &sbd);
-	warn_of_destruction(opts.dev.path);
+	if (rgs == NULL)
+		exit(-1);
+	if (warn_of_destruction(opts.dev.path) != 0)
+		exit(-1);
 
 	if (opts.confirm && !opts.override)
-		are_you_sure();
+		if (!are_you_sure())
+			exit(-1);
 
 	if (!S_ISREG(opts.dev.stat.st_mode) && opts.discard) {
 		if (!opts.quiet) {
@@ -1188,7 +1283,6 @@ int main(int argc, char *argv[])
 		perror(opts.dev.path);
 		exit(EXIT_FAILURE);
 	}
-	posix_fadvise(opts.dev.fd, 0, 0, POSIX_FADV_DONTNEED);
 	error = close(opts.dev.fd);
 	if (error){
 		perror(opts.dev.path);
@@ -1201,3 +1295,4 @@ int main(int argc, char *argv[])
 	}
 	return 0;
 }
+#endif /* UNITTESTS */
